import os
import sys

from struct import pack, unpack
from ReadDataClass import ReadData

FIRMWARE_SIGN = "OWOWOWOWOWOWOWOWOWOWOWOWOWOWOWOW"
MAX_FILENAME_LEN = 0x40

def read_file(filename):
	fd = open(filename, "rb")
	data = fd.read()
	fd.close()
	return data

def write_file(filename, data):
	fd = open(filename, "wb")
	fd.write(data)
	fd.close()

if len(sys.argv) < 2:
	print "Usage: %s <filename>" % __file__
	sys.exit(0)

fw_filename = sys.argv[1]

print "[+] Extracting files from %s" % fw_filename

fw_data = ReadData(read_file(fw_filename), endianness = ">")

print "[+] Searching signature.."

firmware_sign_offset = fw_data.find(FIRMWARE_SIGN)
if  firmware_sign_offset >= 0:
	print "[+] Signature found at offset: %x" % firmware_sign_offset
	
	fw_data.setOffset(firmware_sign_offset)
	print "[+] SIGNATURE: %s" % fw_data.read(len(FIRMWARE_SIGN))
	
	flag = fw_data.readDword()
	file_count = fw_data.readDword()
	
	print "[+] Flag: 0x%08x\n[+] File count: %d" % (flag, file_count)
	
	print "[+] Reading file entries.."
	# parse the file entries
	for i in range(file_count):
		print ">> Entry no %d" % i
		filename = fw_data.read(MAX_FILENAME_LEN)
		print "[+] filename: %s" % filename
		file_size = fw_data.readDword()
		print "[+] file size: %d bytes" % file_size
		file_offset = fw_data.readDword()
		print "[+] file offset: 0x%08x" % file_offset
		
		# extract the data and save it
		f_data = fw_data.readAt(firmware_sign_offset + file_offset, file_size)
		# extract the last part of the path
		write_file(os.path.basename(filename.strip("\x00")), f_data)
		
		fw_data.skipBytes(4) # skip the 'reserved' field
		
		
else:
	print "[!] Signature not found! :("
